/**
 * @file
 * lazy_image_style behaviors.
 */

(function (Drupal, DrupalSettings, debounce) {

  'use strict';

  /**
   * Find lazy img and set src from data-src.
   */
  Drupal.behaviors.lazyImageStyle = {
    attach: function (context, settings) {
      this.init();
      document.addEventListener('scroll', this.init);
      window.addEventListener('resize', this.init);
      window.addEventListener('orientationchange', this.init);
    },

    /**
     * Selector for find lazy images.
     */
    findSelector: 'img[data-src]',

    /**
     * Find and load objects.
     */
    init: debounce(function () {
      let lazyImages = Drupal.behaviors.lazyImageStyle.getObjects();
      Drupal.behaviors.lazyImageStyle.loadObjects(lazyImages);
    }, 250, true),

    /**
     * Return all images supported lazy load.
     *
     * @returns {...NodeListOf<Element>[]}
     */
    getObjects: function () {
      return [...document.querySelectorAll(this.findSelector)];
    },

    /**
     * Load objects.
     *
     * @param {...NodeListOf<Element>[]} elems
     */
    loadObjects: function (elems) {
      let offset = Number(this.getConfig('offset', 100));

      elems.forEach(image => {
        if ((image.getBoundingClientRect().top <= window.innerHeight + offset
            && image.getBoundingClientRect().bottom >= 0) && getComputedStyle(image).display !== 'none') {

          this.setSourceData(image);

          // Remove blue class if found.
          if (image.classList.contains('blue')) {
            image.classList.remove('blue');
          }

          elems = elems.filter(function (elem) {
            return elem !== image;
          });
          if (elems.length === 0) {
            document.removeEventListener('scroll', this.init);
            window.removeEventListener('resize', this.init);
            window.removeEventListener('orientationchange', this.init);
          }
        }
      });
    },

    /**
     * Set source image data.
     */
    setSourceData: function (elem) {
      elem.src = elem.dataset.src;
    },

    /**
     * Return variable from drupal settings.
     *
     * @param key
     * @param default_value
     * @returns {*}
     */
    getConfig: function (key, default_value) {
      let lazy_image_style = DrupalSettings && DrupalSettings.hasOwnProperty('lazy_image_style');

      if (lazy_image_style) {
        return DrupalSettings.lazy_image_style.hasOwnProperty(key) ?
            DrupalSettings.lazy_image_style[key] : default_value;
      }

      return default_value;
    }
  };

}(Drupal, drupalSettings, Drupal.debounce));
