<?php

namespace Drupal\lazy_image_style\Form;

use Drupal\Core\Cache\CacheTagsInvalidator;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Lazy image settings for this site.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * The entity_type.manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * The cache_tags.invalidator service.
   *
   * @var \Drupal\Core\Cache\CacheTagsInvalidator
   */
  protected $cacheTagsInvalidator;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'lazy_image_style_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['lazy_image_style.settings'];
  }

  /**
   * SettingsForm constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Entity\EntityTypeManager $entityTypeManager
   *   The entity type manager service.
   * @param \Drupal\Core\Cache\CacheTagsInvalidator $cacheTagsInvalidator
   *   The cache tags invelidator service.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    EntityTypeManager $entityTypeManager,
    CacheTagsInvalidator $cacheTagsInvalidator
  ) {

    parent::__construct($config_factory);

    $this->entityTypeManager = $entityTypeManager;
    $this->cacheTagsInvalidator = $cacheTagsInvalidator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('cache_tags.invalidator')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('lazy_image_style.settings');

    $image_styles = image_style_options(FALSE);

    $lazy_style_options = array_merge($image_styles, [
      'base64' => $this->t('Use base64'),
      'template_placeholder' => $this->t('Template placeholder'),
    ]);

    $form['lazy_style'] = [
      '#type' => 'select',
      '#title' => $this->t('Temporary style'),
      '#description' => $this->t('This temporary style used before an image is visible.'),
      '#options' => $lazy_style_options,
      '#default_value' => $config->get('lazy_style'),
    ];

    // Description for base64.
    $form['base64_desc'] = [
      '#type' => 'container',
      '#states' => [
        'visible' => [
          ':input[name="lazy_style"]' => ['value' => 'base64'],
        ],
      ],
      [
        '#markup' => $this->t('<strong>Used image white color, 1x1 size and base64 format.</strong>'),
      ]
    ];

    // Description for template_placeholder.
    $form['template_placeholder_desc'] = [
      '#type' => 'container',
      '#states' => [
        'visible' => [
          ':input[name="lazy_style"]' => ['value' => 'template_placeholder'],
        ],
      ],
      [
        '#markup' => $this->t('Testing. <strong>Used <code>lazy_image_tmp</code> theme and <code>lazy-image-tmp.html.twig</code> template.</strong>'),
      ]
    ];

    $form['offset'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Offset'),
      '#default_value' => $config->get('offset') ?: '100',
      '#description' => $this->t("The offset controls how early you want the elements to be loaded before they're visible. Default is <strong>100</strong>, so <strong>100px</strong> before an element is visible it'll start loading."),
      '#field_suffix' => 'px',
      '#maxlength' => 5,
      '#size' => 10,
    ];

    $form['use_blue'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use blue effect'),
      '#default_value' => $config->get('use_blue') ?: FALSE,
      '#states' => [
        'unchecked' => [
          ':input[name="lazy_style"]' => [
            ['value' => 'base64'],
            ['value' => 'template_placeholder'],
          ],
        ],
        'invisible' => [
          ':input[name="lazy_style"]' => [
            ['value' => 'base64'],
            ['value' => 'template_placeholder'],
          ],
        ],
      ],
    ];

    $form['use_group'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Used styles'),
      '#description' => $this->t('Selected style for use lazy image.'),
    ];

    $form['use_group']['use'] = [
      '#type' => 'checkboxes',
      '#options' => $image_styles,
      '#default_value' => $config->get('use') ?: [],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('lazy_image_style.settings');
    $old_used = $config->get('use') ?: [];
    $new_used = array_filter($form_state->getValue('use', []));

    $config->set('use', $new_used)
      ->set('use_blue', $form_state->getValue('use_blue'))
      ->set('lazy_style', $form_state->getValue('lazy_style'))
      ->set('offset', $form_state->getValue('offset'))
      ->save();

    // Invalidate tag cache.
    $tags = array_map(function ($style) {
      return 'lazy_image_style:' . $style;
    }, $old_used + $new_used);
    $this->cacheTagsInvalidator->invalidateTags($tags);

    parent::submitForm($form, $form_state);
  }

}
