<?php

namespace Drupal\lazy_image_style;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\image\ImageStyleInterface;
use Symfony\Component\HttpFoundation\File\MimeType\MimeTypeGuesserInterface;

/**
 * LazyImageStyleHelper service.
 */
class LazyImageStyleHelper {

  CONST TMP_BASE64_IMAGE = "data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAQAAAC1HAwCAAAAC0lEQVR42mNkYAAAAAYAAjCB0C8AAAAASUVORK5CYII=";

  /**
   * The image style.
   *
   * @var \Drupal\image\ImageStyleInterface
   */
  protected $lazy_image_style;

  /**
   * The lazy_image_style ImmutableConfig.
   *
   * @var array
   */
  protected $settings;

  /**
   * The MIME type guesser.
   *
   * @var \Symfony\Component\HttpFoundation\File\MimeType\MimeTypeGuesserInterface
   */
  protected $mimeTypeGuesser;

  /**
   * Constructs a lazyimagestylehelper object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Symfony\Component\HttpFoundation\File\MimeType\MimeTypeGuesserInterface $mimeTypeGuesser
   *   The MIME type guesser.
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    ConfigFactoryInterface $config_factory,
    MimeTypeGuesserInterface $mimeTypeGuesser
  ) {

    $this->settings = $config_factory->get('lazy_image_style.settings');
    $this->mimeTypeGuesser = $mimeTypeGuesser;
    $this->lazy_image_style = $entity_type_manager
      ->getStorage('image_style')
      ->load($this->settings->get('lazy_style'));
  }

  /**
   * Rebuild image render array for lazy load.
   */
  public function rebuild(&$variables) {
    // Check use lazy image for current style.
    if (!$this->useLazyLoad($variables['style_name'])) {
      return;
    }

    // Move source uri to data-src attribute.
    $variables['image']['#attributes']['data-src'] = file_url_transform_relative($variables['image']['#uri']);

    // Attached main library and settings.
    $variables['#attached']['library'][] = 'lazy_image_style/main';
    $variables['#attached']['drupalSettings'] = [
      'lazy_image_style' => [
        'offset' => $this->settings->get('offset') ?: '100',
        'type' => $this->settings->get('type') ?: 'tmp_img',
      ],
    ];

    // Changed theme render if using template_placeholder.
    if ($this->settings->get('lazy_style') == 'template_placeholder') {
      unset($variables['image']['#uri']);
      $variables['image']['#theme'] = 'lazy_image_tmp';
      $variables['#attached']['library'][] = 'lazy_image_style/template';
      return;
    }

    // Set tmp url.
    $variables['image']['#uri'] = $this->getTmpUrl($variables['uri']);

    // Add blue effect.
    if ($this->settings->get('use_blue')) {
      $variables['image']['#attributes']['class'][] = 'blue';
      $variables['#attached']['library'][] = 'lazy_image_style/blue';
    }

  }

  /**
   * Return tmp url image.
   *
   * @param $source_uri
   * @return string
   */
  protected function getTmpUrl($source_uri) {
    $lazy_style = $this->settings->get('lazy_style');

    if ($lazy_style == 'base64') {
      return self::TMP_BASE64_IMAGE;
    }

    if ($this->lazy_image_style instanceof ImageStyleInterface) {
      $tmp_image_uri = $this->lazy_image_style->buildUri($source_uri);

      // Create derivative if necessary.
      if (!file_exists($tmp_image_uri)){
        $this->lazy_image_style->createDerivative($source_uri, $tmp_image_uri);
      }

      $mime = $this->mimeTypeGuesser->guess($tmp_image_uri);
      $base_64_image = base64_encode(file_get_contents($tmp_image_uri));

      return "data:{$mime};base64,{$base_64_image}";
    }

    return self::TMP_BASE64_IMAGE;
  }

  /**
   * Return true if image style use lazy load or false.
   *
   * @param $style_name
   * @return bool
   */
  protected function useLazyLoad($style_name) {
    $used_styles = $this->settings->get('use');
    return is_array($used_styles) && array_key_exists($style_name, $used_styles);
  }

}
